@@ -6,7 +6,7 @@ from paapi5_python_sdk.get_items_request import GetItemsRequest
 from paapi5_python_sdk.get_items_resource import GetItemsResource
 from paapi5_python_sdk.partner_type import PartnerType
 import time
-import logging
+import re
 
 
 REGIONS = {
@@ -47,7 +47,13 @@ class Class:
     """Base class for creating the product instance."""
 
 
-def get_asin(url):
+def _chunks(lst, n):
+    """Yield successive n-sized chunks from lst."""
+    for i in range(0, len(lst), n):
+        yield lst[i:i + n]
+
+
+def get_asin(url: str):
     """Find the ASIN from a given URL.
 
     Args:
@@ -56,13 +62,23 @@ def get_asin(url):
     Returns:
         string: Product ASIN. None if ASIN not found.
     """
-    split_url = url.split('?')[0].replace('?', '/').replace('&', '/').split('/')
-    for c in list(reversed(split_url)):
-        if len(c) == 10 and c.isalnum():
-            return c
+    # Return if url parameter already is the ASIN
+    if re.search(r'^[A-Z0-9]{10}$', url):
+        return url
+    # Extract ASIN from URL searching for alphanumeric and 10 digits
+    have_asin = re.search(r'(dp|gp/product)/([a-zA-Z0-9]{10})', url)
+    return have_asin.group(2) if have_asin else None
 
 
 def parse_product(item):
+    """Parse item data and creates product instance.
+
+    Args:
+        item (instance): The instance with the data from Amazon API.
+
+    Returns:
+        instance: Product instance with parsed data.
+    """
     product = Class()
     product.raw_info = item
 
@@ -266,21 +282,21 @@ def parse_product(item):
         product.images.cropped.large.append(product.images.large.replace('.jpg', '._AC_.jpg'))
     except Exception:
         pass
-    for x in product.images.variants.small:
-        try:
+    try:
+        for x in product.images.variants.small:
             product.images.cropped.small.append(x.replace('_SL', '_AC'))
-        except Exception:
-            pass
-    for x in product.images.variants.medium:
-        try:
+    except Exception:
+        pass
+    try:
+        for x in product.images.variants.medium:
             product.images.cropped.medium.append(x.replace('_SL', '_AC'))
-        except Exception:
-            pass
-    for x in product.images.variants.large:
-        try:
+    except Exception:
+        pass
+    try:
+        for x in product.images.variants.large:
             product.images.cropped.large.append(x.replace('.jpg', '._AC_.jpg'))
-        except Exception:
-            pass
+    except Exception:
+        pass
 
     # Trade In
     product.trade_in = Class()
@@ -520,10 +536,10 @@ class AmazonAPI:
         key (string): Your API key.
         secret (string): Your API secret.
         tag (string): The tag you want to use for the URL.
-        country (string): Country code.
+        country (string): Country code (AU, BR, CA, FR, DE, IN, IT, JP, MX, ES, TR, AE, UK, US).
         throttling (float, optional): Reduce this value to wait longer between API calls.
     """
-    def __init__(self, key, secret, tag, country, throttling=0.9):
+    def __init__(self, key: str, secret: str, tag: str, country: str, throttling=0.9):
         self.key = key
         self.secret = secret
         self.tag = tag
@@ -534,13 +550,13 @@ class AmazonAPI:
         self.marketplace = 'www.amazon.' + DOMAINS[country]
         self.last_query_time = time.time()
 
-    def get_product(self, product_id, condition=Condition.ANY):
+    def get_products(self, product_ids: [str, list], condition=Condition.ANY):
         """Find product information for a specific product on Amazon.
 
         Args:
-            product_id (string): Product ASIN or URL. You can send multiple products separated
-                by commas.
-            condition (class, optional): Specify the product condition. Defaults to NEW.
+            product_ids (string): One or more item ids like ASIN or product URL (max 10).
+            Could be a string separated by comma or as a list.
+            condition (class, optional): Specify the product condition. Defaults to ANY.
 
         Returns:
             class instance: An instance of the class Product containing all the available
@@ -553,10 +569,14 @@ class AmazonAPI:
                          host=self.host,
                          region=self.region)
 
-        product_id = product_id.split(',')
-        asin_list = []
-        for x in product_id:
-            asin_list.append(get_asin(x.strip()))
+        # Clean up input data into a list stripping any extra white space
+        asin_or_url_list = [x.strip() for x in product_ids.split(",")] if isinstance(product_ids, str) else product_ids
+
+        # Extract ASIN if supplied input is product URL and remove any duplicate ASIN
+        asin_full_list = list(set([get_asin(x) for x in asin_or_url_list]))
+
+        # Creates lists of 10 items each
+        asin_full_list = list(_chunks(asin_full_list, 10))
 
         product_resources = [
             GetItemsResource.BROWSENODEINFO_BROWSENODES,
@@ -615,38 +635,35 @@ class AmazonAPI:
             GetItemsResource.RENTALOFFERS_LISTINGS_DELIVERYINFO_SHIPPINGCHARGES,
             GetItemsResource.RENTALOFFERS_LISTINGS_MERCHANTINFO]
 
-        try:
-            request = GetItemsRequest(partner_tag=self.tag,
-                                      partner_type=PartnerType.ASSOCIATES,
-                                      marketplace=self.marketplace,
-                                      condition=condition,
-                                      item_ids=asin_list,
-                                      resources=product_resources)
-        except Exception as exception:
-            raise exception
+        results = []
+        for asin_list in asin_full_list:
+            try:
+                request = GetItemsRequest(partner_tag=self.tag,
+                                          partner_type=PartnerType.ASSOCIATES,
+                                          marketplace=self.marketplace,
+                                          condition=condition,
+                                          item_ids=asin_list,
+                                          resources=product_resources)
+            except Exception as exception:
+                raise exception
 
-        try:
-            # Wait before doing the request
-            wait_time = 1 / self.throttling - (time.time() - self.last_query_time)
-            if wait_time > 0:
-                time.sleep(wait_time)
-            self.last_query_time = time.time()
+            try:
+                # Wait before doing the request
+                wait_time = 1 / self.throttling - (time.time() - self.last_query_time)
+                if wait_time > 0:
+                    time.sleep(wait_time)
+                self.last_query_time = time.time()
 
-            response = api.get_items(request)
-            if response.items_result is not None:
-                if len(response.items_result.items) > 0:
-                    results = []
-                    for item in response.items_result.items:
-                        product = parse_product(item)
-                        results.append(product)
-                    if len(results) == 0:
-                        return None
-                    elif len(results) == 1:
-                        return results[0]
-                    else:
-                        return results
-                else:
-                    return None
+                response = api.get_items(request)
+                if response.items_result is not None:
+                    if len(response.items_result.items) > 0:
+                        for item in response.items_result.items:
+                            product = parse_product(item)
+                            results.append(product)
+            except Exception as exception:
+                raise exception
 
-        except Exception as exception:
-            raise exception
+        if results:
+            return results
+        else:
+            return None
